package Plugins::Unique;
use Config::General;

my $plugin_name = 'Unique';

my $conf = new Config::General(
    -ConfigFile      => $config::plugins_dir . '/' . $plugin_name . '.conf',
    -InterPolateVars => TRUE,
    -AutoLaunder     => TRUE
);
my %cfg = $conf->getall;
my $ignore_val = {};
my $unique_att = {};

my $VERSION  = 1.2;
my $this_pkg = (caller(0))[0] . '::' . $plugin_name;

$Plugins::Unique::panic      = $cfg{'panic_on_errors'};
@Plugins::Unique::attributes = keys %{ $cfg{'attributes'} };
@Plugins::Unique::ignore     = keys %{ $cfg{'ignore'} };

AOSICFG::write_ws_log('-=- ' . $this_pkg . ': panic on errors=' . $Plugins::Unique::panic) if $config::aosi_debug_level > 1;
AOSICFG::write_ws_log('-=- ' . $this_pkg . ': getting attributes list: ' . join(", ", @Plugins::Unique::attributes)) if $config::aosi_debug_level > 1;
foreach my $u_attr (@Plugins::Unique::attributes) {
    $unique_att->{$u_attr}++;
}
AOSICFG::write_ws_log('-=- ' . $this_pkg . ': getting ignore list: ' . join(", ", @Plugins::Unique::ignore)) if $config::aosi_debug_level > 1;
foreach my $i_attr (@Plugins::Unique::ignore) {
    $ignore_val->{$i_attr} = $cfg{'ignore'}->{$i_attr};
    AOSICFG::write_ws_log('-=- ' . $this_pkg . ': ignore: ' . $i_attr. '=' .  $cfg{'ignore'}->{$i_attr}) if $config::aosi_debug_level > 1;
}

##################################################################
# Check
##################################################################
sub checkEntry {
    my ($ldap, $base_dn, $entry_in, $user, $pwd) = @_;
    my ($this_fun,  $msg,         $code,      $out_panic);
    my ($ent_num,   $ent_attrs,   $attr,        $att,       $vals, $val, $unique, $hash, $result, $dn);
    my ($entries,   $attr_exists, $user_base, $uid, $mesg);
    $this_fun  = (caller(0))[3];
    $msg       = '';
    $code      = 0;
    $out_panic = 0;

    AOSICFG::write_ws_log("|> $this_fun(in): base_dn=$base_dn;user=$user;entry_in=$entry_in;") if $config::aosi_debug_level > 1;
    $dn = $entry_in->{'dn'};
    ($uid, $user_base) = AOSICFG::get_uid_base($dn);
    $ent_attrs = $entry_in->{'attrs'};
    AOSICFG::write_ws_log("|> $this_fun(checking for user): user=$uid;base=$base_dn;attributes=".AOSICFG::dump_array($ent_attrs)) if $config::aosi_debug_level > 1;

    foreach $attr (@$ent_attrs) {    # Fetch some attributes
        $att  = $attr->{'att'};
        $vals = $attr->{'val'};
        foreach $val (@$vals) {
            if($unique_att->{$att}){
                AOSICFG::write_ws_log("|> $this_fun: Attribute $att is on unique list") if $config::aosi_debug_level > 3;
                if($ignore_val->{$att} ne $val){
                    $filter = '(&(' . $att . '=' . $val . ')(!(uid=' . $uid . ')))';

                    AOSICFG::write_ws_log("|> $this_fun(looking for): $att=$val;filter=$filter;") if $config::aosi_debug_level > 3;
					$mesg    = $ldap->bind($config::ldap_user, password => AOSICFG::get_pwd($config::ldap_pwd_file), version => 3);
					$mesg    = $ldap->search(base => $base_dn, filter => $filter, timelimit => 60, attrs => ['uid']);
					$code2   = $mesg->code;
					$error2  = $mesg->error;
					$entries = $mesg->{'entries'};
                    $ent_num = $#{$entries};
                    AOSICFG::write_ws_log("|> $this_fun: #entries=" . $ent_num) if $config::aosi_debug_level > 3;
                    if ($ent_num > -1) {
						my $e_uid = $entries->[0]->get_value('uid');
                    	AOSICFG::write_ws_log("|> $this_fun(found uid): $e_uid") if $config::aosi_debug_level > 3;
                        $msg .= 'User with UID '
                          . $e_uid
                          . ' has the same '
                          . $att . ' ('
                          . $val
                          . ') attribute! ';
                        $code      = -10601;
                        $out_panic = $Plugins::Unique::panic;
                        last;
                    }
                }
                else {
                    AOSICFG::write_ws_log("|> $this_fun: $att=$val is ignored;") if $config::aosi_debug_level > 1;
                }
            }
        }
    }
    $msg = $msg eq '' ? 'OK' : $msg;

    AOSICFG::write_ws_log('|> ' . $this_fun . '(out): code=' . $code . ';msg=' . $msg) if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

##################################################################
#
##################################################################
sub beforeAddUser {
    my ($status_hash, $ldap, $base_dn, $entry_in, $user, $pwd) = @_;
    my ($this_fun, $msg, $code, $out_panic);
    $this_fun  = (caller(0))[3];
    $msg       = '';
    $code      = 0;
    $out_panic = 0;

    AOSICFG::write_ws_log("|> $this_fun(in): base_dn=$base_dn;user=$user;entry_in=$entry_in;") if $config::aosi_debug_level > 1;

    ($code, $msg, $out_panic) = checkEntry($ldap, $base_dn, $entry_in, $user, $pwd);

    AOSICFG::write_ws_log('|> ' . $this_fun . '(out): code=' . $code . ';msg=' . $msg) if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

##################################################################
#
##################################################################
sub afterAddUser {
    my ($status_hash, $entry_in, $out_xml, $user, $pwd) = @_;
    my ($this_fun, $msg,       $code, $out_panic);
    my ($dn,       $ent_attrs, $attr, $search);
    $this_fun  = (caller(0))[3];
    $msg       = 'OK';
    $code      = 0;
    $out_panic = 0;

    AOSICFG::write_ws_log("|> $this_fun(in): base_dn=$base_dn;user=$user;entry_in=$entry_in;") if $config::aosi_debug_level > 1;
    AOSICFG::write_ws_log('|> ' . $this_fun . '(out): code=' . $code . ';msg=' . $msg)         if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

##################################################################
#  Check if adding non-unique value
##################################################################
# Same function for all actions
# 1 = beforeAddAttribute
# 2 = beforeModifyAttribute
# 3 = beforeDeleteAttribute
sub beforeChangeAttribute {
    my ($status_hash, $type, $ldap, $base_dn, $entry_in, $user, $pwd) = @_;
    my ($this_fun, $msg, $code, $out_panic);
    $this_fun  = (caller(0))[3];
    $msg       = '';
    $code      = 0;
    $out_panic = 0;

    AOSICFG::write_ws_log("|> $this_fun(in): tpye=$type;ldap=$ldap;base=$base_dn;user=$user;entry_in=$entry_in;") if $config::aosi_debug_level > 1;

    ($code, $msg, $out_panic) = checkEntry($ldap, $base_dn, $entry_in, $user, $pwd);

    AOSICFG::write_ws_log('|> ' . $this_fun . '(out): code=' . $code . ';msg=' . $msg) if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

##################################################################
#  Change attributes
##################################################################
# Same function for all actions
# 1 = afterAddAttribute
# 2 = afterModifyAttribute
# 3 = afterDeleteAttribute
sub afterChangeAttribute {
    my ($status_hash, $type, $entry_in, $out_xml, $user, $pwd) = @_;
    my ($this_fun, $out_panic, $msg);
    $this_fun  = (caller(0))[3];
    $msg       = 'OK';
    $code      = 0;
    $out_panic = 0;

    AOSICFG::write_ws_log("|> $this_fun(in): tpye=$type;user=$user;entry_in=$entry_in;") if $config::aosi_debug_level > 1;

    AOSICFG::write_ws_log('|> ' . $this_fun . '(out): code=' . $code . ';msg=' . $msg) if $config::aosi_debug_level > 1;
    return $code, $msg, $out_panic;
}

1;
